@@ -33,7 +33,7 @@
 #include "debug.h"
 #include <vte/vte.h>
 #include "vteaccess.h"
-#include "vte-private.h"
+#include "vteinternal.hh"
 
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
@@ -712,7 +712,8 @@ vte_terminal_accessible_initialize (AtkObject *obj, gpointer data)
 
 	ATK_OBJECT_CLASS (_vte_terminal_accessible_parent_class)->initialize (obj, data);
 
-	_vte_terminal_accessible_ref(terminal);
+        auto impl = IMPL(terminal);
+        impl->subscribe_accessible_events();
 
 	g_signal_connect(terminal, "text-inserted",
 			 G_CALLBACK(vte_terminal_accessible_text_modified),
@@ -1407,8 +1408,9 @@ vte_terminal_accessible_get_selection(AtkText *text, gint selection_number,
         VteTerminalAccessible *accessible = VTE_TERMINAL_ACCESSIBLE(text);
 	VteTerminalAccessiblePrivate *priv = (VteTerminalAccessiblePrivate *)_vte_terminal_accessible_get_instance_private(accessible);
 	GtkWidget *widget;
-	VteTerminal *terminal;
-	long start_x, start_y, end_x, end_y;
+
+	if (selection_number != 0)
+		return NULL;
 
 	vte_terminal_accessible_update_private_data_if_needed(accessible,
 							      NULL, NULL);
@@ -1418,20 +1420,18 @@ vte_terminal_accessible_get_selection(AtkText *text, gint selection_number,
 		return NULL;
 	}
 
-	terminal = VTE_TERMINAL (widget);
-	if (!vte_terminal_get_has_selection (terminal)) {
-		return NULL;
-	}
-	if (selection_number != 0) {
-		return NULL;
-	}
+        auto impl = IMPL_FROM_WIDGET(widget);
 
-	_vte_terminal_get_start_selection (terminal, &start_x, &start_y);
+	if (!impl->m_has_selection)
+		return NULL;
 
-	*start_offset = offset_from_xy (priv, start_x, start_y);
-	_vte_terminal_get_end_selection (terminal, &end_x, &end_y);
-	*end_offset = offset_from_xy (priv, end_x, end_y);
-	return _vte_terminal_get_selection (terminal);
+        auto start_sel = impl->m_selection_start;
+        auto end_sel = impl->m_selection_end;
+
+	*start_offset = offset_from_xy (priv, start_sel.col, start_sel.row);
+	*end_offset = offset_from_xy (priv, end_sel.col, end_sel.row);
+
+	return g_strdup(impl->m_selection_text[VTE_SELECTION_PRIMARY]);
 }
 
 static gboolean
@@ -1441,8 +1441,6 @@ vte_terminal_accessible_add_selection(AtkText *text,
         VteTerminalAccessible *accessible = VTE_TERMINAL_ACCESSIBLE(text);
 	VteTerminalAccessiblePrivate *priv = (VteTerminalAccessiblePrivate *)_vte_terminal_accessible_get_instance_private(accessible);
 	GtkWidget *widget;
-	VteTerminal *terminal;
-	gint start_x, start_y, end_x, end_y;
 
 	vte_terminal_accessible_update_private_data_if_needed(accessible,
 							      NULL, NULL);
@@ -1452,12 +1450,10 @@ vte_terminal_accessible_add_selection(AtkText *text,
 		return FALSE;
 	}
 
-	terminal = VTE_TERMINAL (widget);
-	g_assert(!vte_terminal_get_has_selection (terminal));
-
+	int start_x, start_y, end_x, end_y;
 	xy_from_offset (priv, start_offset, &start_x, &start_y);
 	xy_from_offset (priv, end_offset, &end_x, &end_y);
-	_vte_terminal_select_text (terminal, start_x, start_y, end_x, end_y, start_offset, end_offset);
+	IMPL_FROM_WIDGET(widget)->select_text(start_x, start_y, end_x, end_y);
 	return TRUE;
 }
 
@@ -1573,19 +1569,24 @@ vte_terminal_accessible_set_size(AtkComponent *component,
 				 gint width, gint height)
 {
         VteTerminalAccessible *accessible = VTE_TERMINAL_ACCESSIBLE(component);
-	VteTerminal *terminal;
-	long columns, rows;
 	GtkWidget *widget;
 
 	widget = gtk_accessible_get_widget (GTK_ACCESSIBLE(accessible));
-	if (widget == NULL) {
+	if (widget == NULL)
 		return FALSE;
-	}
-	terminal = VTE_TERMINAL(widget);
+
+        VteTerminal *terminal = VTE_TERMINAL(widget);
+        auto impl = IMPL(terminal);
 
         /* If the size is an exact multiple of the cell size, use that,
          * otherwise round down. */
-        (void) _vte_terminal_size_to_grid_size(terminal, width, height, &columns, &rows);
+        width -= impl->m_padding.left + impl->m_padding.right;
+        height -= impl->m_padding.top + impl->m_padding.bottom;
+
+        auto columns = width / impl->m_char_width;
+        auto rows = height / impl->m_char_height;
+        if (columns <= 0 || rows <= 0)
+                return FALSE;
 
 	vte_terminal_set_size(terminal, columns, rows);
 	return (vte_terminal_get_row_count (terminal) == rows) &&
